/*
 * Copyright (c) 2008, Witold Wysota
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright notice,
 *       this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * The name of Witold Wysota may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <QtCore>
#include <QtGui>
#include <QtNetwork>


class Widget : public QWidget {
    Q_OBJECT
public:
    Widget() : QWidget(){
      QVBoxLayout *l = new QVBoxLayout(this);
      QLabel *label = new QLabel(tr("Enter url and press return"));
      l->addWidget(label);
      lineEdit = new QLineEdit;
      lineEdit->setText("http://www.qtcentre.org");
      l->addWidget(lineEdit);
      textBrowser = new QTextBrowser;
      l->addWidget(textBrowser);
      connect(lineEdit, SIGNAL(returnPressed()), this, SLOT(download()));
  }
protected slots:
    void download(){
        QString url = lineEdit->text();
        QNetworkAccessManager manager;
        QEventLoop q;
        QTimer tT;
        tT.setSingleShot(true);
        connect(&tT, SIGNAL(timeout()), &q, SLOT(quit()));
        connect(&manager, SIGNAL(finished(QNetworkReply*)), &q, SLOT(quit()));
        QNetworkReply *reply = manager.get(QNetworkRequest(QUrl(url)));
        tT.start(5000); // 5s timeout
        lineEdit->setEnabled(false);
        q.exec();
        if(tT.isActive()){
            // download complete
            tT.stop();
            textBrowser->setHtml(reply->readAll());
            lineEdit->setEnabled(true);
            delete reply;
            return ;
        } else {
            // timeout
            delete reply;
            lineEdit->setEnabled(true);
            return ;
        }
    }
private:
    QTextBrowser *textBrowser;
    QLineEdit *lineEdit;
};

#include "main.moc"

int main(int argc, char **argv){
    QApplication app(argc, argv);
    Widget w;
    w.show();
    return app.exec();
}
